import unittest
import json
import tempfile
import pathlib
import PIL.Image
import torch
from irisml.tasks.export_coco_from_detection_predictions import Task


class FakeDataset(torch.utils.data.Dataset):
    def __init__(self, data):
        self._data = data

    def __len__(self):
        return len(self._data)

    def __getitem__(self, index):
        return self._data[index]


class TestExportCOCOFromDetectionPredictions(unittest.TestCase):
    def test_simple(self):
        width = height = 10
        od_dataset = FakeDataset([(PIL.Image.new('RGB', (width, height)), [[i, 0, 0, 0.5, 0.5]]) for i in range(2)])
        detections = [torch.tensor([[0, 0.9, 0, 0, 0.5, 0.5]], dtype=float), torch.tensor([[1, 0.2, 0, 0, 0.3, 0.3], [2, 0.1, 0.3, 0.3, 0.6, 0.6]], dtype=float)]

        with tempfile.TemporaryDirectory() as temp_dir:
            temp_dir = pathlib.Path(temp_dir)
            filepath = temp_dir / 'detections.json'
            Task(Task.Config(filepath)).execute(Task.Inputs(dataset=od_dataset, detections=detections))
            coco = json.loads(filepath.read_text())
            self.assertDictEqual(coco[0], {"image_id": 1, "category_id": 1, "score": 0.9, "bbox": [0., 0., 5., 5.]})
            self.assertDictEqual(coco[1], {"image_id": 2, "category_id": 2, "score": 0.2, "bbox": [0., 0., 3., 3.]})
            self.assertDictEqual(coco[2], {"image_id": 2, "category_id": 3, "score": 0.1, "bbox": [3., 3., 3., 3.]})



import dataclasses
import typing
import pathlib
import json
import torch
import torch.utils.data
import irisml.core


class Task(irisml.core.TaskBase):
    """Export OD detections to COCO file.

    The OD detections is a list of tensor, with each tensor row being [category_id, score, x, y, x, y] normalized.
    The JSON file is a list of dictionaries, each dictionary has key "image_id", "bbox" xywh normalized, "category_id", and "score". All indices are supposed to start from 1.
    """

    VERSION = '0.1.0'
    CACHE_ENABLED = False

    @dataclasses.dataclass
    class Inputs:
        dataset: torch.utils.data.Dataset
        detections: typing.List[torch.Tensor]

    @dataclasses.dataclass
    class Config:
        filepath: pathlib.Path = 'detections.json'

    def execute(self, inputs):
        def _convert_bbox(bbox: list, w: int, h: int):
            return [bbox[0] * w, bbox[1] * h, (bbox[2] - bbox[0]) * w, (bbox[3] - bbox[1]) * h]

        dataset = inputs.dataset
        coco = []
        detections = [torch.tensor(d) for d in inputs.detections]

        for idx, dts_img in enumerate(detections):
            image = dataset[idx][0]
            w, h = (image.shape[1], image.shape[0]) if isinstance(image, torch.Tensor) else image.size
            for dt in dts_img:
                coco.append({
                    "category_id": int(dt[0].item()) + 1,
                    "image_id": idx + 1,
                    "bbox": _convert_bbox(dt[2:].tolist(), w, h),
                    "score": dt[1].item()
                })

        pathlib.Path(self.config.filepath).write_text(json.dumps(coco, indent=2))
        return self.Outputs()
